/*
 * Interrupt handling.
 *
 * Copyright (C) 2015, 2016 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "board-defs.h"

#include "init.h"
#include "board.h"
#include "cpu.h"
#include "irq.h"
#include "tasks.h"

/* Initialisation and handling. */

void irq_init()
{
	handle_error_level();
	timer_init_irq();
	init_interrupts();
}

void irq_handle()
{
	unsigned short i;

	/* Check interrupt identity. */

	if (REG_INTC_IPR & (1 << TIMER_CHAN_IRQ)) {

		/* Switch task. */

		switch_task();

		/* Clear interrupt status. */

		timer_clear();

	/* Handle other interrupts, anyway. */

	} else {
		for (i = 0; i < 32; i++) {
			if (REG_INTC_IPR & (1 << i))
				__intc_ack_irq(i);
		}
	}
}
